#picaxe 14m2

symbol LED = PINB.2
symbol LED_MASK_PIEZO = %00000100 ; mask for pin 2
symbol LED_PWM = B.2

symbol PIEZO = B.1

symbol KEY = PINC.4
symbol KEY_PULLUP = 0x1000 ; mask for C.4
symbol KEY_DEBOUNCE_TIME = 20 ; debounce time for key
; key is pressed if pin is LOW
#define KEY_PRESSED KEY = 0 


SYMBOL OLED_WIDTH	= 128
SYMBOL OLED_MAX_X	= OLED_WIDTH - 1 ; 127
SYMBOL OLED_HEIGHT = 64
SYMBOL OLED_HEIGHT_LINES	= OLED_HEIGHT / 8 ; 8
SYMBOL OLED_MAX_Y	= OLED_HEIGHT_LINES - 1 ; 7

SYMBOL SSD1306_SETCONTRAST = 0x81
SYMBOL SSD1306_DISPLAYALLON_RESUME = 0xA4
SYMBOL SSD1306_DISPLAYALLON  = 0xA5
SYMBOL SSD1306_NORMALDISPLAY = 0xA6
SYMBOL SSD1306_INVERTDISPLAY = 0xA7
SYMBOL SSD1306_DISPLAYOFF = 0xAE
SYMBOL SSD1306_DISPLAYON = 0xAF	
SYMBOL SSD1306_SETDISPLAYOFFSET = 0xD3
SYMBOL SSD1306_SETCOMPINS = 0xDA	
SYMBOL SSD1306_SETVCOMDETECT = 0xDB	
SYMBOL SSD1306_SETDISPLAYCLOCKDIV = 0xD5
SYMBOL SSD1306_SETPRECHARGE = 0xD9
SYMBOL SSD1306_SETMULTIPLEX = 0xA8	
SYMBOL SSD1306_SETLOWCOLUMN = 0x00
SYMBOL SSD1306_SETHIGHCOLUMN = 0x10
SYMBOL SSD1306_SETSTARTLINE = 0x40
SYMBOL SSD1306_MEMORYMODE = 0x20
SYMBOL SSD1306_COLUMNADDR = 0x21
SYMBOL SSD1306_PAGEADDR = 0x22		; Page 0-7 represents line 0 - 7
SYMBOL SSD1306_COMSCANINC = 0xC0
SYMBOL SSD1306_COMSCANDEC = 0xC8
SYMBOL SSD1306_SEGREMAP_0 = 0xA0
SYMBOL SSD1306_SEGREMAP_1 = 0xA0 | 1
SYMBOL SSD1306_CHARGEPUMP = 0x8D
SYMBOL SSD1306_EXTERNALVCC = 0x1
SYMBOL SSD1306_SWITCHCAPVCC = 0x2


symbol PICAXE_SPEED = m32
symbol I2C_SPEED = i2cfast_32

SYMBOL SSD1306_ADDR  = 0x3C * 2 ; OLED I2C address (0x78)

symbol IMAGE_DELAY = 1000*8 ; 1000ms @ 32 MHz speed

symbol SSP1STAT_SFR = 0x94 ; address of SFR register SSP1STAT, needed for pokesfr

symbol oled_x1 = b0
symbol oled_y1 = b1
symbol oled_x2 = b2
symbol oled_y2 = b3
symbol tempVarWord = w4


#macro i2c_enable_oled 
hi2csetup i2cmaster, SSD1306_ADDR, I2C_SPEED, i2cbyte
pokesfr SSP1STAT_SFR, 0x40 ; enable SMBus mode => 2,1V on I2C pins is recognized as HIGH level
#endmacro


#macro OLED_CURSOR(x,y) ; x = column (0..127), y = line (0..7)
oled_y1 = y
oled_x1 = x
oled_y2 = OLED_MAX_Y
oled_x2 = OLED_MAX_X
gosub oledWindow
#endmacro


#macro OLED_WINDOW(x1,y1,x2,y2) ; x1=start column (0..127), y1=start line (0..7)
                                ; x2=end column (0..127), y2=end line (0..7)
oled_y1 = y1
oled_x1 = x1
oled_y2 = y2
oled_x2 = x2
gosub oledWindow
#endmacro


#define OLED_DATA(d) hi2cout (0x40, d)

;----------------------------------------------------------------

setfreq PICAXE_SPEED

gosub oledInit
gosub oledClearDisplay

main:
pullup KEY_PULLUP ; activate internal pullup for key detection

prog_loop:

OLED_CURSOR(0, 0)

; start of logo ***********************************************************************
;// 'make_logo_128x64', 128x64px
hi2cout (0x40,_
0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x80, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xc0, 0xc0, _
0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xf0, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xf8, 0xff, 0xff, 0xff, 0xff, _
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x07, 0x3f, 0xff, 0xff, 0xff, 0xff, 0xff, _
0xf8, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0xfc, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x0f, _
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x30, 0xf8, 0xf8, 0xfc, 0xfc, _
0x7e, 0x7e, 0x7e, 0x3e, 0x3f, 0x3f, 0x3f, 0x3f, 0x7f, 0x7f, 0xff, 0xfe, 0xfe, 0xfe, 0xfc, 0xf8, _
0xe0, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x80, 0xc0, _
0xe0, 0xf0, 0xf8, 0xfe, 0xfe, 0xfe, 0xfe, 0x3e, 0x1e, 0x0e, 0x06, 0x02, 0xc0, 0xe0, 0xf0, 0xf8, _
0xfc, 0xfc, 0xfe, 0xfe, 0x7e, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x3f, 0x7e, 0xfe, 0xfe, 0xfc, 0xf8, _
0xf0, 0xe0, 0x80, 0x00, 0x00, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0xfe, 0x00, 0x00, _
0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x03, 0x1f, 0xff, 0xff, 0xff, _
0xff, 0xff, 0xfc, 0xe0, 0x00, 0x00, 0xe0, 0xfe, 0xff, 0xff, 0xff, 0xff, 0x3f, 0x03, 0x00, 0x00, _
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc0, 0xe1, 0xe1, 0xf0, _
0xf0, 0xf8, 0xf8, 0xf8, 0x78, 0x78, 0x7c, 0x7c, 0x3c, 0x3c, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, _
0xff, 0xfe, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xfc, 0xfe, 0xff, 0xff, _
0xff, 0xff, 0xff, 0xff, 0xf3, 0xe1, 0x80, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xff, 0xff, 0xff, 0xff, _
0xff, 0xff, 0xff, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf8, 0xf9, 0xff, 0xff, 0xff, _
0xff, 0xff, 0xff, 0xfc, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x3f, _
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x7f, 0x0f, 0x01, 0x00, 0x00, 0x00, 0x00, _
0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x3c, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, _
0xff, 0xe1, 0xe0, 0xc0, 0xc0, 0xe0, 0xe0, 0xe0, 0xf0, 0xf0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, _
0xff, 0xff, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x07, 0x03, 0x01, 0x01, _
0x07, 0x1f, 0x7f, 0xff, 0xff, 0xff, 0xff, 0xfe, 0xf8, 0xe0, 0x80, 0x0f, 0x3f, 0x7f, 0xff, 0xff, _
0xff, 0xff, 0xff, 0xf0, 0xe0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xe0, 0xe0, 0xf0, 0xf0, _
0xf0, 0xe0, 0xc0, 0x00, 0x00, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0xf0, 0x00, 0x00, _
0x00, 0x00, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x01, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0xc3, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, _
0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0x07, 0x07, 0x07, _
0x07, 0x07, 0x07, 0x07, 0x07, 0x03, 0x03, 0x01, 0x01, 0x00, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, _
0x07, 0x07, 0x00, 0x00, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x00, 0x00, 0x00, 0x00, _
0x00, 0x00, 0x00, 0x00, 0x03, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x04, 0x00, 0xc0, 0xc1, _
0x03, 0x03, 0x07, 0x07, 0x07, 0x07, 0x0f, 0x0f, 0x0f, 0x0f, 0x07, 0x07, 0x07, 0x07, 0x07, 0x03, _
0x03, 0x01, 0x00, 0x00, 0xc0, 0xc7, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x07, 0x00, 0x00, _
0x00, 0x00, 0xfe, 0xfe, 0x0c, 0x04, 0x06, 0xfe, 0xfc, 0x04, 0x06, 0xfe, 0xfc, 0x00, 0x00, 0xc0, _
0xe4, 0x96, 0x96, 0x96, 0xfc, 0xf8, 0x00, 0x00, 0xff, 0xff, 0x30, 0x38, 0x7c, 0xc6, 0x00, 0x00, _
0x78, 0xfc, 0xb6, 0xb6, 0x36, 0xbe, 0x9c, 0x00, 0x00, 0x30, 0x30, 0x30, 0x30, 0x00, 0x00, 0xfe, _
0xfe, 0x0c, 0x04, 0x06, 0xfe, 0xfc, 0x04, 0x06, 0xfe, 0xfc, 0x00, 0x00, 0xc0, 0xe4, 0x96, 0x96, _
0x96, 0xfc, 0xf8, 0x00, 0x58, 0xfc, 0xa6, 0xa2, 0xa2, 0x3e, 0x1c, 0x06, 0x00, 0xc0, 0xe4, 0x96, _
0x96, 0x96, 0xfc, 0xf8, 0x00, 0x00, 0xc6, 0xe6, 0xb6, 0x9e, 0x8e, 0x80, 0x00, 0x00, 0xfe, 0xfe, _
0x00, 0x00, 0xfe, 0xfe, 0x04, 0x06, 0x06, 0xfe, 0xfc, 0x00, 0x00, 0x80, 0x80, 0x00, 0x70, 0xfc, _
0x8e, 0x86, 0x84, 0xcc, 0xff, 0xff, 0x00, 0x78, 0xfc, 0xb6, 0xb6, 0x36, 0xbe, 0x9c, 0x00, 0x00, _
0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, _
0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, _
0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, _
0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, _
0x00, 0x01, 0x01, 0x00, 0x06, 0x07, 0x09, 0x09, 0x09, 0x05, 0x07, 0x02, 0x00, 0x00, 0x01, 0x01, _
0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, _
0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, _
0x01, 0x01, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00  _
)
; end of logo ***********************************************************************

pause IMAGE_DELAY

; clear upper left quarter of OLED
OLED_WINDOW(0,0,63,3)
for tempVarWord = 1 to 256 ; 64*32/8
   OLED_DATA(0)
next tempVarWord
pause 2000

; clear upper right quarter of OLED
OLED_WINDOW(64,0,127,3)
for tempVarWord = 1 to 256 ; 64*32/8
   OLED_DATA(0)
next tempVarWord
pause 2000

; clear lower right quarter of OLED
OLED_WINDOW(64,4,127,7)
for tempVarWord = 1 to 256 ; 64*32/8
   OLED_DATA(0)
next tempVarWord
pause 2000

; clear lower left quarter of OLED
OLED_WINDOW(0,4,63,7)
for tempVarWord = 1 to 256 ; 64*32/8
   OLED_DATA(0)
next tempVarWord
pause 8000

goto prog_loop

;-----------------------------------------------------------------

oledInit:
   i2c_enable_oled
   hi2cout (0, SSD1306_DISPLAYOFF)            ; 0xAE
   hi2cout (0, SSD1306_SETDISPLAYCLOCKDIV)    ; 0xD5
   hi2cout (0, 0x80)                          ; the suggested ratio 0x80
   hi2cout (0, SSD1306_SETMULTIPLEX)          ; 0xA8
   hi2cout (0, 0x3F)
   hi2cout (0, SSD1306_SETDISPLAYOFFSET)      ; 0xD3
   hi2cout (0, 0x0)                           ; no offset
   hi2cout (0, SSD1306_SETSTARTLINE)          ; line #0
   hi2cout (0, SSD1306_CHARGEPUMP)            ; 0x8D
   hi2cout (0, 0x14)     						    ; INTERNAL VCC
   hi2cout (0, SSD1306_MEMORYMODE)            ; 0x20
   hi2cout (0, 0x00)                          ; Horiz mode. 0x0 act like ks0108
   ;hi2cout (0, SSD1306_SEGREMAP_1)
   ;hi2cout (0, SSD1306_COMSCANDEC)
   hi2cout (0, SSD1306_SEGREMAP_0)            ; Bild auf OLED horizontal spiegeln
   hi2cout (0, SSD1306_COMSCANINC)            ; Bild auf OLED vertikal spiegeln
   hi2cout (0, SSD1306_SETCOMPINS)            ; 0xDA
   hi2cout (0, 0x12)
   hi2cout (0, SSD1306_SETCONTRAST)           ; 0x81
   hi2cout (0, 0xCF)						          ; INTERNAL VCC
   hi2cout (0, SSD1306_SETPRECHARGE)          ; 0xD9
   hi2cout (0, 0xF1)						          ; INTERNAL VCC
   hi2cout (0, SSD1306_SETVCOMDETECT)         ; 0xDB
   hi2cout (0, 0x40)
   hi2cout (0, SSD1306_DISPLAYALLON_RESUME)   ; 0xA4
   hi2cout (0, SSD1306_DISPLAYON)         	 ; 0xAF	
return


oledWindow:
   hi2cout (0, SSD1306_PAGEADDR, oled_y1, oled_y2) ; set row
   hi2cout (0, SSD1306_COLUMNADDR, oled_x1, oled_x2)  ; set column
return


oledClearDisplay:
   hi2cout (0, SSD1306_PAGEADDR, 0, OLED_MAX_Y) ; set row 0..7
   hi2cout (0, SSD1306_COLUMNADDR, 0, OLED_MAX_X) ; set column 0..127
   for tempVarWord = 1 to 32 ; 1024/32
   	hi2cout (0x40, _
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, _
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0  _
         )
   next tempVarWord
return

